import java.util.*;
import java.io.*;

/**
 * Основной класс. Он же пускалка. Хранит информацию об адресной книге, умеет добавлять в эту
 * книгу записи (а также их удалять и редактировать).
 */
public class AddressBook {
    private SortedMap<String, Contact> _contactsByName = new TreeMap<String,Contact>();

    public AddressBook() {
        try {
            ObjectInputStream inputStream = new ObjectInputStream(new FileInputStream("contacts.dat"));
            _contactsByName = (SortedMap<String, Contact>) inputStream.readObject();
            inputStream.close();
        } catch (Exception e) {
            _contactsByName = new TreeMap<String,Contact>();
        }
    }

    public SortedMap<String, Contact> getContactsByName() {
        return _contactsByName;
    }

    public Contact getContactByNumber(String aNumber) {
        for (Map.Entry<String, Contact> entry : _contactsByName.entrySet()) {
            if (entry.getValue().getNumber().equals(aNumber)) {
                return entry.getValue();
            }
        }

        return null;
    }

    /**
     * Сохраняет контакты в файл contacts.xml (в директории запуска),
     * используя XStream. Файл создается, если его нет.
     * @throws IOException Выбрасывается, если появились проблемы записи файла
     */
    private void saveContactsInformation() throws IOException {
        ObjectOutputStream outputStream = new ObjectOutputStream(new FileOutputStream("contacts.dat"));
        outputStream.writeObject(_contactsByName);
        outputStream.close();
    }

    /**
     * Добавляет контакт в адресную книгу
     * @param aName Имя
     * @param aNumber Телефонный номер
     * @param aComment Комментарий
     * @throws IOException Выбрасывается, если появились проблемы записи файла
     */
    public void addContact(String aName, String aNumber, String aComment) throws IOException {
        Contact contact = new Contact();
        contact.setData(aName, aNumber, aComment);
        _contactsByName.put(aName, contact);
        saveContactsInformation();
    }

    /**
     * Удаляет контакт по телефонному номеру.
     * @param aNumber телефонный номер, который определяет удаляемый контакт.
     * @throws IOException Выбрасывается, если появились проблемы записи файла
     */
    public void removeContactByNumber(String aNumber) throws IOException {
        Contact contact = getContactByNumber(aNumber);
        if (contact != null) {
            _contactsByName.remove(contact.getName());
            saveContactsInformation();
        }
    }

    /**
     * Редактирует контакт, если такой контакт существует.
     * @param aEditNumber Номер, который определяет редактируемый контакт.
     * @param aName Новое имя контакта.
     * @param aNumber Новый номер контакта.
     * @param aComment Новый комментарий для контакта.
     * @throws IOException Выбрасывается, если появились проблемы записи файла
     */
    public void editContact(String aEditNumber, String aName, String aNumber, String aComment) throws IOException {
        Contact contact = getContactByNumber(aEditNumber);
        if (contact != null) {
            contact.setData(aName, aNumber, aComment);
            saveContactsInformation();
        }
    }
}
