import javax.servlet.http.*;
import javax.servlet.*;
import java.io.*;
import java.util.*;

public class AddressBookServlet extends HttpServlet {
    private AddressBook _addressBook = null;

    public void init(ServletConfig aServletConfig) throws ServletException {
        super.init(aServletConfig);
        _addressBook = new AddressBook();
    }

    protected void doGet(HttpServletRequest aRequest, HttpServletResponse aResponse) throws ServletException, IOException {
        handle(aRequest, aResponse);
    }

    protected void doPost(HttpServletRequest aRequest, HttpServletResponse aResponse) throws ServletException, IOException {
        handle(aRequest, aResponse);
    }

    private void handle(HttpServletRequest aRequest, HttpServletResponse aResponse) throws ServletException, IOException {
        aRequest.setCharacterEncoding("utf-8");

        String target = aRequest.getRequestURI().substring(aRequest.getContextPath().length());

        if (target.equals("/")) {
            outputPage("index.jsp", aRequest, aResponse);
        } else if ("/add".equals(target)) {
            handleAdd(aRequest, aResponse);
        } else if ("/view".equals(target)) {
            handleView(aRequest, aResponse);
        } else if ("/edit".equals(target)) {
            handleEdit(aRequest, aResponse);
        } else if ("/remove".equals(target)) {
            handleRemove(aRequest, aResponse);
        }
    }

    private void handleRemove(HttpServletRequest aRequest, HttpServletResponse aResponse) throws IOException, ServletException {
        if (aRequest.getParameter("number") != null) {
            _addressBook.removeContactByNumber(aRequest.getParameter("number"));
            aRequest.setAttribute("message", "Удален контакт с номером \"" + aRequest.getParameter("number") + "\"");
        }

        handleView(aRequest, aResponse);
    }

    private void handleEdit(HttpServletRequest aRequest, HttpServletResponse aResponse) throws IOException, ServletException {
        if (aRequest.getParameter("number") == null) {
            _addressBook.removeContactByNumber(aRequest.getParameter("number"));
            aRequest.setAttribute("message", "Не определено, что редактировать");
            handleView(aRequest, aResponse);
        } else if (aRequest.getParameter("edited") != null) {
            _addressBook.editContact(aRequest.getParameter("edited"),
                    aRequest.getParameter("name"), aRequest.getParameter("number"), aRequest.getParameter("comment"));
            aRequest.setAttribute("message", "Контакт \"" + aRequest.getParameter("name") + "\" отредактирован");
            handleView(aRequest, aResponse);
        } else {
            Contact contact = _addressBook.getContactByNumber(aRequest.getParameter("number"));
            aRequest.setAttribute("action", "edit");
            aRequest.setAttribute("edit.name", contact.getName());
            aRequest.setAttribute("edit.number", contact.getNumber());
            aRequest.setAttribute("edit.comment", contact.getComment());
            outputPage("edit.jsp", aRequest, aResponse);
        }
    }

    private void handleAdd(HttpServletRequest aRequest, HttpServletResponse aResponse) throws IOException, ServletException {
        if (aRequest.getParameter("name") != null) {
            _addressBook.addContact(aRequest.getParameter("name"), aRequest.getParameter("number"), aRequest.getParameter("comment"));
            aRequest.setAttribute("message", "Контакт \"" + aRequest.getParameter("name") + "\" добавлен");
            handleView(aRequest, aResponse);
        } else {
            aRequest.setAttribute("action", "add");
            aRequest.setAttribute("edit.name", "");
            aRequest.setAttribute("edit.number", "");
            aRequest.setAttribute("edit.comment", "");
            outputPage("edit.jsp", aRequest, aResponse);
        }
    }

    private void handleView(HttpServletRequest aRequest, HttpServletResponse aResponse) throws IOException, ServletException {
        Map<String, String> numbers = new HashMap<String, String>();
        Map<String, String> comments = new HashMap<String, String>();

        for (Map.Entry<String, Contact> entry : _addressBook.getContactsByName().entrySet()) {
            numbers.put(entry.getKey(), entry.getValue().getNumber());
            comments.put(entry.getKey(), entry.getValue().getComment());
        }

        aRequest.setAttribute("numbers", numbers);
        aRequest.setAttribute("comments", comments);

        if (aRequest.getAttribute("message") == null) {
            aRequest.setAttribute("message", "");
        }

        outputPage("view.jsp", aRequest, aResponse);
    }

    public void outputPage(String aJSPName, HttpServletRequest aRequest, HttpServletResponse aResponse) throws IOException, ServletException {
        RequestDispatcher dispatcher = aRequest.getRequestDispatcher("/jsps/" + aJSPName);
        dispatcher.forward(aRequest, aResponse);
    }
}
